%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The model solution is log-linear. This has two implications for the 
% output of stoch_simul:
%   (i)  if a variable, say x_t, is simulated, then you get x_t
%   (ii) if you want the impulse response of x_t to a shock, say s, then
%        what you find in x_s is (x_t - x)/x, where x is the steady state 
%        of x_t.
% The following transformations reflect these beliefs about the Dynare 
% output.
%
% the following computes standard deviations of the variables and variance decompositions
% [vardecomp,labels,z]=disp_th_moments(oo_.dr,options_.varlist);
% these are the rows that are of interest: tt=[5 [28:38]];

% following computes the variance decomposition
% @# include "getvardecomp.mod"
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

ia=3;
ib=3;

idx = 1:16;
% I want the response of the log level of a variable to a shock, in log 
% deviation from what it would have been if there had been no shock.
yr      = 100 * cumsum(gdp_obs_e_sigma);
cr      = 100 * cumsum(consumption_obs_e_sigma);
ir      = 100 * cumsum(investment_obs_e_sigma);
lrr     = 100 * cumsum(rL_e_sigma);

subplot(2, 2, 1)
plot(idx, yr(idx))
axis tight
subplot(2, 2, 2)
plot(idx, cr(idx))
axis tight
subplot(2, 2, 3)
plot(idx, ir(idx))
axis tight
subplot(2, 2, 4)
plot(idx, lrr(idx))
axis tight

Rk_p = 0.020209405759659;
creditr = 100 * cumsum(credit_obs_e_sigma);
netwr   = 100 * cumsum(networth_obs_e_sigma);
sloper  = 40000 * Spread1_obs_e_sigma;
premr   = 40000 * premium_obs_e_sigma;
riskr   = sigma_e_sigma;
inflr   = 400 * inflation_obs_e_sigma;
Rkv     = 400 * (Rk_p * Rk_e_sigma - Re_p * Re_e_sigma);
%
%% computing the log-linearized 40 period rate....
%tt   = 1:(length(yr) - 1);
%t1   = 2:length(yr);
%rsyn = zeros(length(tt), 1);
%for ii = 1:(length(tt) - 39);
%  rsyn(ii) = mean(RealRe_obs_e_sigma(ii:(ii+39)));
%end
%
%xx = [yr(tt), cr(tt), ir(tt), 100 * hours_obs_e_sigma(tt), ...
%      400 * RealRe_obs_e_sigma(tt), 400 * inflation_obs_e_sigma(tt), ... 
%      400 * Re_p * Re_e_sigma(tt), ...
%      400 * (Re_p * Re_e_sigma(tt) - inflation_obs_e_sigma(t1)), ... 
%      400 * (muzstar_p / beta_p) * rL_e_sigma(tt), ...
%      400 * (muzstar_p / beta_p) * rsyn];
%
%xx(1:16,:)
%
gg = 1:16;
ss = 0:(length(gg) - 1);
%
%if exist('consumption_obs_e_xi8') == 0;
%    return;
%end;
%
cr8      = 100 * cumsum(consumption_obs_e_xi8);
yr8      = 100 * cumsum(gdp_obs_e_xi8);
ir8      = 100 * cumsum(investment_obs_e_xi8);
creditr8 = 100 * cumsum(credit_obs_e_xi8);
netwr8   = 100 * cumsum(networth_obs_e_xi8);
sloper8  = 40000 * Spread1_obs_e_xi8;
premr8   = 40000 * premium_obs_e_xi8;
riskr8   = sigma_e_xi8;
inflr8   = 400 * inflation_obs_e_xi8;
%Rk8      = 400 * (Rk_p * Rk_e_xi8 - Re_p * Re_e_xi8);
%
%
[Yr, I] = min(yr);
%
yrmei         = 100 * cumsum(gdp_obs_e_zetai);
[Yrmei, I]    = min(yrmei);
yrmei         = yrmei * Yr / Yrmei;
yrequity      = -100 * cumsum(gdp_obs_e_gamma);
[Yrequity, I] = min(yrequity);
yrequity      = yrequity * Yr / Yrequity;
%
crmei      = 100 * cumsum(consumption_obs_e_zetai) * Yr / Yrmei;
irmei      = 100 * cumsum(investment_obs_e_zetai) * Yr / Yrmei;
creditrmei = 100 * cumsum(credit_obs_e_zetai) * Yr / Yrmei;
netwrmei   = 100 * cumsum(networth_obs_e_zetai) * Yr / Yrmei;
slopermei  = 40000 * Spread1_obs_e_zetai * Yr / Yrmei;
premrmei   = 40000 * premium_obs_e_zetai * Yr / Yrmei;
riskrmei   = sigma_e_zetai * Yr / Yrmei;
inflrmei   = 400 * inflation_obs_e_zetai * Yr / Yrmei;
Rkmei      = 400 * (Rk_p * Rk_e_zetai - Re_p * Re_e_zetai) ...
                 * Yr / Yrmei;
%
crequity      = -100 * cumsum(consumption_obs_e_gamma) * Yr / Yrequity;
irequity      = -100 * cumsum(investment_obs_e_gamma) * Yr / Yrequity;
creditrequity = -100 * cumsum(credit_obs_e_gamma) * Yr / Yrequity;
netwrequity   = -100 * cumsum(networth_obs_e_gamma) * Yr / Yrequity;
sloperequity  = -40000 * Spread1_obs_e_gamma * Yr / Yrequity;
premrequity   = -40000 * premium_obs_e_gamma * Yr / Yrequity;
riskrequity   = -sigma_e_gamma * Yr / Yrequity;
inflrequity   = -400 * inflation_obs_e_gamma * Yr / Yrequity;
Rkequity      = -400 * (Rk_p * Rk_e_gamma - Re_p * Re_e_gamma) ...
                    * Yr / Yrequity;
%
figure('name', 'other irfs')
subplot(ia, ib, 6)
plot(ss, cr(gg), ss, crmei(gg), 'o-', ss, crequity(gg), '*-')
title('F: consumption')
axis tight
subplot(ia, ib, 4)
plot(ss, yr(gg), ss, yrmei(gg), 'o-', ss, yrequity(gg), '*-')
title('D: output')
axis tight
subplot(ia, ib, 1)
plot(ss, premr(gg), ss, premrmei(gg), 'o-', ss, premrequity(gg), '*-')
title('A: interest rate spread (Annual Basis Points)')
legend('unanticipated risk shock, \xi_{0,0}', ...
       'innovation in marginal efficiency of investment, \zeta _{It}',...
       'negative innovation in equity shock, \gamma_{t}')
axis tight
subplot(ia, ib, 3)
plot(ss, ir(gg), ss, irmei(gg), 'o-', ss, irequity(gg), '*-')
title('C: investment')
axis tight
subplot(ia, ib, 2)
plot(ss, creditr(gg), ss, creditrmei(gg), 'o-', ...
     ss, creditrequity(gg), '*-')
title('B: credit')
axis tight
subplot(ia, ib, 5)
plot(ss, netwr(gg), ss, netwrmei(gg), 'o-', ss, netwrequity(gg), '*-')
title('E: net worth')
axis tight
subplot(ia, ib, 7)
plot(ss, inflr(gg), ss, inflrmei(gg), 'o-', ss, inflrequity(gg), '*-')
title('G: inflation (APR)')
axis tight
subplot(ia, ib, 8)
plot(ss, Rkv(gg), ss, Rkmei(gg), 'o-', ss, Rkequity(gg), '*-')
title('H: excess return on capital, R^{k}-R (APR)')
axis tight
subplot(ia, ib, 9)
plot(ss, sloper(gg), ss, slopermei(gg), 'o-', ss, sloperequity(gg), '*-')
title('H: slope of term structure (Annual Basis Points)')
axis tight
str=['Figure 6: Dynamic Responses to Three Shocks'];
hout=suptitle(str);

