function [S_all, S_risk] = get_var_decomp_freq(xparam1)
%
% function dynare_estimation(var_list)
% runs the estimation of the model
%  
% INPUTS
%   var_list:  selected endogenous variables vector
%  
% OUTPUTS
%   none
%
% SPECIAL REQUIREMENTS
%   none

% Copyright (C) 2003-2011 Dynare Team
%
% This file is part of Dynare.
%
% Dynare is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% Dynare is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
var_list = [];

global options_ oo_ M_ oo_recursive_ estim_params_ bayestopt_

options_.qz_criterium=1;

var_list = check_list_of_variables(options_, M_, var_list);
options_.varlist = var_list;

%&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
%taken from dynare_estimation_1
%

%% Set various options.
options_ = set_default_option(options_,'nodiagnostic',0);

%% Set number of observations
%% Set the number of observed variables.
n_varobs = size(options_.varobs,1);

%taken from setprior.m
nvx = size(estim_params_.var_exo,1);
nvn = size(estim_params_.var_endo,1);
ncx = size(estim_params_.corrx,1);
ncn = size(estim_params_.corrn,1);
np = size(estim_params_.param_vals,1);

estim_params_.nvx = nvx;
estim_params_.nvn = nvn;
estim_params_.ncx = ncx;
estim_params_.ncn = ncn;
estim_params_.np = np;

%% Get the number of parameters to be estimated. 
nvx = estim_params_.nvx;  % Variance of the structural innovations (number of parameters).
nvn = estim_params_.nvn;  % Variance of the measurement innovations (number of parameters).
ncx = estim_params_.ncx;  % Covariance of the structural innovations (number of parameters).
ncn = estim_params_.ncn;  % Covariance of the measurement innovations (number of parameters).
np  = estim_params_.np ;  % Number of deep parameters.
nx  = nvx+nvn+ncx+ncn+np; % Total number of parameters to be estimated.

dr = oo_.dr;

%% Compute the steady state: 
if ~isempty(estim_params_)
    set_parameters(xparam1);
end
if options_.steadystate_flag% if the *_steadystate.m file is provided.
    [ys,tchek] = feval([M_.fname '_steadystate'],...
                       [zeros(M_.exo_nbr,1);...
                        oo_.exo_det_steady_state]);
    if size(ys,1) < M_.endo_nbr 
        if length(M_.aux_vars) > 0
            ys = add_auxiliary_variables_to_steadystate(ys,M_.aux_vars,...
                                                        M_.fname,...
                                                        zeros(M_.exo_nbr,1),...
                                                        oo_.exo_det_steady_state,...
                                                        M_.params,...
                                                        options_.bytecode);
        else
            error([M_.fname '_steadystate.m doesn''t match the model']);
        end
    end
    oo_.steady_state = ys;
else% if the steady state file is not provided.
    [dd,info] = resol(oo_.steady_state,0);
    oo_.steady_state = dd.ys; clear('dd');
end

% taken from dynare_estimation_init
%% Setting resticted state space (observed + predetermined variables)
var_obs_index = [];
k1 = [];
for i=1:n_varobs
    var_obs_index = [var_obs_index strmatch(deblank(options_.varobs(i,:)),M_.endo_names(dr.order_var,:),'exact')];
    k1 = [k1 strmatch(deblank(options_.varobs(i,:)),M_.endo_names, 'exact')];
end
% Define union of observed and state variables
k2 = union(var_obs_index',[dr.nstatic+1:dr.nstatic+dr.npred]', 'rows');
% Set restrict_state to postion of observed + state variables in expanded state vector.
oo_.dr.restrict_var_list = k2;
% set mf0 to positions of state variables in restricted state vector for likelihood computation.
[junk,bayestopt_.mf0] = ismember([dr.nstatic+1:dr.nstatic+dr.npred]',k2);
% Set mf1 to positions of observed variables in restricted state vector for likelihood computation.
[junk,bayestopt_.mf1] = ismember(var_obs_index,k2); 
% Set mf2 to positions of observed variables in expanded state vector for filtering and smoothing.
bayestopt_.mf2  = var_obs_index;
bayestopt_.mfys = k1;

dr = set_state_space(oo_.dr,M_);
oo_.dr = dr;
nstatic = dr.nstatic;          % Number of static variables. 
npred = dr.npred;              % Number of predetermined variables.
nspred = dr.nspred;            % Number of predetermined variables in the state equation.


[junk,ic] = intersect(k2,nstatic+(1:npred)');
oo_.dr.restrict_columns = [ic; length(k2)+(1:nspred-npred)'];


%taken from DSGElikelihood.m

fval            = [];
ys              = [];
trend_coeff     = [];
cost_flag       = 1;
%------------------------------------------------------------------------------
% 1. Get the structural parameters & define penalties
%------------------------------------------------------------------------------
Q = M_.Sigma_e;
H = M_.H;
for i=1:estim_params_.nvx
    k =estim_params_.var_exo(i,1);
    Q(k,k) = xparam1(i)*xparam1(i);
end
offset = estim_params_.nvx;
if estim_params_.nvn
    for i=1:estim_params_.nvn
        k = estim_params_.var_endo(i,1);
        H(k,k) = xparam1(i+offset)*xparam1(i+offset);
    end
    offset = offset+estim_params_.nvn;
end
if estim_params_.ncx
    for i=1:estim_params_.ncx
        k1 =estim_params_.corrx(i,1);
        k2 =estim_params_.corrx(i,2);
        Q(k1,k2) = xparam1(i+offset)*sqrt(Q(k1,k1)*Q(k2,k2));
        Q(k2,k1) = Q(k1,k2);
    end
    [CholQ,testQ] = chol(Q);
    if testQ    %% The variance-covariance matrix of the structural innovations is not definite positive.
        %% We have to compute the eigenvalues of this matrix in order to build the penalty.
        a = diag(eig(Q));
        k = find(a < 0);
        if k > 0
            fval = bayestopt_.penalty+sum(-a(k));
            cost_flag = 0;
            info = 43;
            return
        end
    end
    offset = offset+estim_params_.ncx;
end
if estim_params_.ncn 
    for i=1:estim_params_.ncn
        k1 = options_.lgyidx2varobs(estim_params_.corrn(i,1));
        k2 = options_.lgyidx2varobs(estim_params_.corrn(i,2));
        H(k1,k2) = xparam1(i+offset)*sqrt(H(k1,k1)*H(k2,k2));
        H(k2,k1) = H(k1,k2);
    end
    [CholH,testH] = chol(H);
    if testH
        a = diag(eig(H));
        k = find(a < 0);
        if k > 0
            fval = bayestopt_.penalty+sum(-a(k));
            cost_flag = 0;
            info = 44;
            return
        end
    end
    offset = offset+estim_params_.ncn;
end
if estim_params_.np > 0
    M_.params(estim_params_.param_vals(:,1)) = xparam1(offset+1:end);
end
M_.Sigma_e = Q;
M_.H = H;
%------------------------------------------------------------------------------
% 2. call model setup & reduction program
%------------------------------------------------------------------------------
[T,R,SteadyState,info] = dynare_resolve('restrict');
Q_all=Q;
Q_risk=zeros(20,20);
ss=[10 [13:20]];
Q_risk(ss,ss)=Q(ss,ss);

N2=250;
jj=1:N2;
om=pi/N2*jj;
[omh,Ih]=min(abs(om-2*pi/8));
[oml,Il]=min(abs(om-2*pi/32));
i=sqrt(-1);
z=exp(-i*om);
I=eye(size(T,1));
S_all=zeros(size(T,1),size(T,1));
S_risk=zeros(size(T,1),size(T,1));

for ii = Il:Ih
    d=(1-z(ii));
    d=d*conj(d);
    A=I-T*z(ii);
    Ai=inv(A);
    %MATLAB treats the prime as transposition and conjugation
    SS_all=Ai*R*Q_all*R'*Ai'/d;
    SS_risk=Ai*R*Q_risk*R'*Ai'/d;
    S_all=SS_all+S_all;
    S_risk=SS_risk+S_risk;
end
S_all=S_all/N2;
S_risk=S_risk/N2;

